import { BaseRepository } from './base-repository';
import { PrintHistory, Step } from '@shared/index';

export class InMemoryRepository<T extends { id: number }> {
  private items: T[] = [];
  private nextId = 1;

  async findAll(): Promise<T[]> {
    return [...this.items];
  }

  async findById(id: number): Promise<T | null> {
    return this.items.find(item => item.id === id) || null;
  }

  async create(data: Omit<T, 'id' | 'created_at' | 'updated_at'>): Promise<T> {
    const now = new Date();
    const item = {
      ...data,
      id: this.nextId++,
      created_at: now,
      updated_at: now,
    } as unknown as T;
    this.items.push(item);
    return item;
  }

  async update(id: number, data: Partial<T>): Promise<boolean> {
    const idx = this.items.findIndex(item => item.id === id);
    if (idx === -1) return false;
    this.items[idx] = { ...this.items[idx], ...data, updated_at: new Date() };
    return true;
  }

  async delete(id: number): Promise<boolean> {
    const idx = this.items.findIndex(item => item.id === id);
    if (idx === -1) return false;
    this.items.splice(idx, 1);
    return true;
  }
}

export class InMemoryPrintHistoryRepository extends InMemoryRepository<PrintHistory> {
  async findByTaskId(taskId: number): Promise<PrintHistory[]> {
    return (await this.findAll()).filter(ph => ph.task_id === taskId);
  }
  async findByStepId(stepId: number): Promise<PrintHistory[]> {
    return (await this.findAll()).filter(ph => ph.step_id === stepId);
  }
}

export class InMemoryStepRepository extends InMemoryRepository<Step> {
  async findByTaskId(taskId: number): Promise<Step[]> {
    return (await this.findAll()).filter(step => step.task_id === taskId);
  }
  async incrementPrintCount(id: number): Promise<boolean> {
    const step = await this.findById(id);
    if (!step) return false;
    return this.update(id, {
      print_count: (step.print_count || 0) + 1,
      last_printed_at: new Date(),
    });
  }
}