import { jest } from '@jest/globals';
import { Knex } from 'knex';
import { testDb } from '../../db/__tests__/setup';
import { resolvers } from '../resolvers';
import { GroupRepository, TaskRepository, StepRepository, UserRepository, NoteRepository, PrintHistoryRepository, ImageRepository } from '../../db/repositories';
import type { Printer, Task, Step } from '@shared/index';

describe('GraphQL Resolvers', () => {
  const context = {
    db: testDb,
    printer: {
      printTask: jest.fn(),
      printStep: jest.fn(),
    } as unknown as Printer,
  };

  const groupRepo = new GroupRepository(testDb);
  const taskRepo = new TaskRepository(testDb);
  const stepRepo = new StepRepository(testDb);
  const userRepo = new UserRepository(testDb);
  const noteRepo = new NoteRepository(testDb);
  const printHistoryRepo = new PrintHistoryRepository(testDb);
  const imageRepo = new ImageRepository(testDb);

  beforeEach(async () => {
    await testDb('print_history').del();
    await testDb('notes').del();
    await testDb('images').del();
    await testDb('steps').del();
    await testDb('tasks').del();
    await testDb('groups').del();
    await testDb('users').del();
  });

  describe('Queries', () => {
    describe('groups', () => {
      it('should return all groups', async () => {
        await groupRepo.create({ name: 'Group 1' });
        await groupRepo.create({ name: 'Group 2' });

        const result = await resolvers.Query.groups(null, null, context);
        expect(result).toHaveLength(2);
        expect(result[0]?.name).toBe('Group 1');
        expect(result[1]?.name).toBe('Group 2');
      });
    });

    describe('group', () => {
      it('should return a group by id', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        const result = await resolvers.Query.group(null, { id: group.id.toString() }, context);
        expect(result?.name).toBe('Test Group');
      });

      it('should return null for non-existent group', async () => {
        const result = await resolvers.Query.group(null, { id: '999' }, context);
        expect(result).toBeNull();
      });
    });

    describe('tasks', () => {
      it('should return tasks for a group', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        await taskRepo.create({ name: 'Task 1', group_id: group.id, print_count: 0 });
        await taskRepo.create({ name: 'Task 2', group_id: group.id, print_count: 0 });

        const result = await resolvers.Query.tasks(null, { groupId: group.id.toString() }, context);
        expect(result).toHaveLength(2);
        expect(result[0]?.name).toBe('Task 1');
        expect(result[1]?.name).toBe('Task 2');
      });
    });

    describe('task', () => {
      it('should return a task by id', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        const task = await taskRepo.create({ name: 'Test Task', group_id: group.id, print_count: 0 });
        const result = await resolvers.Query.task(null, { id: task.id.toString() }, context);
        expect(result?.name).toBe('Test Task');
      });

      it('should return null for non-existent task', async () => {
        const result = await resolvers.Query.task(null, { id: '999' }, context);
        expect(result).toBeNull();
      });
    });

    describe('steps', () => {
      it('should return steps for a task', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        const task = await taskRepo.create({ name: 'Test Task', group_id: group.id, print_count: 0 });
        await stepRepo.create({ name: 'Step 1', instructions: 'Instructions 1', task_id: task.id, order: 1, print_count: 0 });
        await stepRepo.create({ name: 'Step 2', instructions: 'Instructions 2', task_id: task.id, order: 2, print_count: 0 });

        const result = await resolvers.Query.steps(null, { taskId: task.id.toString() }, context);
        expect(result).toHaveLength(2);
        expect(result[0]?.name).toBe('Step 1');
        expect(result[1]?.name).toBe('Step 2');
      });
    });

    describe('step', () => {
      it('should return a step by id', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        const task = await taskRepo.create({ name: 'Test Task', group_id: group.id, print_count: 0 });
        const step = await stepRepo.create({
          name: 'Test Step',
          instructions: 'Test Instructions',
          task_id: task.id,
          order: 1,
          print_count: 0
        });

        const result = await resolvers.Query.step(null, { id: step.id.toString() }, context);
        expect(result?.name).toBe('Test Step');
      });

      it('should return null for non-existent step', async () => {
        const result = await resolvers.Query.step(null, { id: '999' }, context);
        expect(result).toBeNull();
      });
    });

    describe('notes', () => {
      it('should return notes for a task', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        const task = await taskRepo.create({ name: 'Test Task', group_id: group.id, print_count: 0 });
        const user = await userRepo.create({ name: 'Test User' });
        await noteRepo.create({ content: 'Note 1', task_id: task.id, created_by: user.id });
        await noteRepo.create({ content: 'Note 2', task_id: task.id, created_by: user.id });

        const result = await resolvers.Query.notes(null, { taskId: task.id.toString() }, context);
        expect(result).toHaveLength(2);
        expect(result[0]?.content).toBe('Note 1');
        expect(result[1]?.content).toBe('Note 2');
      });

      it('should return notes for a step', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        const task = await taskRepo.create({ name: 'Test Task', group_id: group.id, print_count: 0 });
        const step = await stepRepo.create({
          name: 'Test Step',
          instructions: 'Test Instructions',
          task_id: task.id,
          order: 1,
          print_count: 0
        });
        const user = await userRepo.create({ name: 'Test User' });
        await noteRepo.create({ content: 'Note 1', step_id: step.id, created_by: user.id });
        await noteRepo.create({ content: 'Note 2', step_id: step.id, created_by: user.id });

        const result = await resolvers.Query.notes(null, { stepId: step.id.toString() }, context);
        expect(result).toHaveLength(2);
        expect(result[0]?.content).toBe('Note 1');
        expect(result[1]?.content).toBe('Note 2');
      });

      it('should return empty array for non-existent task', async () => {
        const result = await resolvers.Query.notes(null, { taskId: '999' }, context);
        expect(result).toHaveLength(0);
      });

      it('should return empty array for non-existent step', async () => {
        const result = await resolvers.Query.notes(null, { stepId: '999' }, context);
        expect(result).toHaveLength(0);
      });
    });

    describe('printHistory', () => {
      it('should return print history for a task', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        const task = await taskRepo.create({ name: 'Test Task', group_id: group.id, print_count: 0 });
        const user = await userRepo.create({ name: 'Test User' });
        await printHistoryRepo.create({
          user_id: user.id,
          task_id: task.id,
          printed_at: new Date(),
        });
        await printHistoryRepo.create({
          user_id: user.id,
          task_id: task.id,
          printed_at: new Date(),
        });

        const result = await resolvers.Query.printHistory(null, { taskId: task.id.toString() }, context);
        expect(result).toHaveLength(2);
      });

      it('should return print history for a step', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        const task = await taskRepo.create({ name: 'Test Task', group_id: group.id, print_count: 0 });
        const step = await stepRepo.create({
          name: 'Test Step',
          instructions: 'Test Instructions',
          task_id: task.id,
          order: 1,
          print_count: 0
        });
        const user = await userRepo.create({ name: 'Test User' });
        await printHistoryRepo.create({
          user_id: user.id,
          step_id: step.id,
          printed_at: new Date(),
        });
        await printHistoryRepo.create({
          user_id: user.id,
          step_id: step.id,
          printed_at: new Date(),
        });

        const result = await resolvers.Query.printHistory(null, { stepId: step.id.toString() }, context);
        expect(result).toHaveLength(2);
      });

      it('should return empty array for non-existent task', async () => {
        const result = await resolvers.Query.printHistory(null, { taskId: '999' }, context);
        expect(result).toHaveLength(0);
      });

      it('should return empty array for non-existent step', async () => {
        const result = await resolvers.Query.printHistory(null, { stepId: '999' }, context);
        expect(result).toHaveLength(0);
      });
    });
  });

  describe('Mutations', () => {
    describe('createGroup', () => {
      it('should create a group', async () => {
        const result = await resolvers.Mutation.createGroup(null, { name: 'Test Group' }, context);
        expect(result?.name).toBe('Test Group');
      });

      it('should create a group with parent', async () => {
        const parent = await groupRepo.create({ name: 'Parent Group' });
        const result = await resolvers.Mutation.createGroup(
          null,
          { name: 'Child Group', parentId: parent.id.toString() },
          context
        );
        expect(result.name).toBe('Child Group');
        expect(result.parent_id).toBe(parent.id);
      });

      it('should throw error for non-existent parent', async () => {
        await expect(
          resolvers.Mutation.createGroup(null, { name: 'Child Group', parentId: '999' }, context)
        ).rejects.toThrow('Parent group not found');
      });
    });

    describe('createTask', () => {
      it('should create a task', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        const result = await resolvers.Mutation.createTask(
          null,
          { name: 'Test Task', groupId: group.id.toString() },
          context
        );
        expect(result.name).toBe('Test Task');
        expect(result.group_id).toBe(group.id);
      });

      it('should throw error for non-existent group', async () => {
        await expect(
          resolvers.Mutation.createTask(null, { name: 'Test Task', groupId: '999' }, context)
        ).rejects.toThrow('Group not found');
      });
    });

    describe('createStep', () => {
      it('should create a step', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        const task = await taskRepo.create({ name: 'Test Task', group_id: group.id, print_count: 0 });
        const result = await resolvers.Mutation.createStep(
          null,
          {
            name: 'Test Step',
            instructions: 'Test Instructions',
            taskId: task.id.toString(),
            order: 1,
          },
          context
        );
        expect(result.name).toBe('Test Step');
        expect(result.task_id).toBe(task.id);
      });

      it('should throw error for non-existent task', async () => {
        await expect(
          resolvers.Mutation.createStep(
            null,
            {
              name: 'Test Step',
              instructions: 'Test Instructions',
              taskId: '999',
              order: 1,
            },
            context
          )
        ).rejects.toThrow('Task not found');
      });
    });

    describe('createImage', () => {
      it('should create an image', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        const task = await taskRepo.create({ name: 'Test Task', group_id: group.id, print_count: 0 });
        const step = await stepRepo.create({
          name: 'Test Step',
          instructions: 'Test Instructions',
          task_id: task.id,
          order: 1,
          print_count: 0
        });
        const result = await resolvers.Mutation.createImage(
          null,
          {
            stepId: step.id.toString(),
            originalPath: '/path/to/image.jpg',
            bwPath: '/path/to/image.bw.jpg',
            order: 1,
          },
          context
        );
        expect(result.original_path).toBe('/path/to/image.jpg');
        expect(result.bw_path).toBe('/path/to/image.bw.jpg');
        expect(result.step_id).toBe(step.id);
      });

      it('should throw error for non-existent step', async () => {
        await expect(
          resolvers.Mutation.createImage(
            null,
            {
              stepId: '999',
              originalPath: '/path/to/image.jpg',
              bwPath: '/path/to/image.bw.jpg',
              order: 1,
            },
            context
          )
        ).rejects.toThrow('Step not found');
      });
    });

    describe('createNote', () => {
      it('should create a note for a task', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        const task = await taskRepo.create({ name: 'Test Task', group_id: group.id, print_count: 0 });
        const user = await userRepo.create({ name: 'Test User' });
        const result = await resolvers.Mutation.createNote(
          null,
          {
            content: 'Test Note',
            taskId: task.id.toString(),
            userId: user.id.toString(),
          },
          context
        );
        expect(result.content).toBe('Test Note');
        expect(result.task_id).toBe(task.id);
        expect(result.created_by).toBe(user.id);
      });

      it('should create a note for a step', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        const task = await taskRepo.create({ name: 'Test Task', group_id: group.id, print_count: 0 });
        const step = await stepRepo.create({
          name: 'Test Step',
          instructions: 'Test Instructions',
          task_id: task.id,
          order: 1,
          print_count: 0
        });
        const user = await userRepo.create({ name: 'Test User' });
        const result = await resolvers.Mutation.createNote(
          null,
          {
            content: 'Test Note',
            stepId: step.id.toString(),
            userId: user.id.toString(),
          },
          context
        );
        expect(result.content).toBe('Test Note');
        expect(result.step_id).toBe(step.id);
        expect(result.created_by).toBe(user.id);
      });

      it('should throw error for non-existent task', async () => {
        const user = await userRepo.create({ name: 'Test User' });
        await expect(
          resolvers.Mutation.createNote(
            null,
            {
              content: 'Test Note',
              taskId: '999',
              userId: user.id.toString(),
            },
            context
          )
        ).rejects.toThrow('Task not found');
      });

      it('should throw error for non-existent step', async () => {
        const user = await userRepo.create({ name: 'Test User' });
        await expect(
          resolvers.Mutation.createNote(
            null,
            {
              content: 'Test Note',
              stepId: '999',
              userId: user.id.toString(),
            },
            context
          )
        ).rejects.toThrow('Step not found');
      });

      it('should throw error for non-existent user', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        const task = await taskRepo.create({ name: 'Test Task', group_id: group.id, print_count: 0 });
        const step = await stepRepo.create({
          name: 'Test Step',
          instructions: 'Test Instructions',
          task_id: task.id,
          order: 1,
          print_count: 0
        });
        await expect(
          resolvers.Mutation.createNote(
            null,
            {
              content: 'Test Note',
              stepId: step.id.toString(),
              userId: '999',
            },
            context
          )
        ).rejects.toThrow('User not found');
      });
    });

    describe('printTask', () => {
      it('should print a task', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        const task = await taskRepo.create({ name: 'Test Task', group_id: group.id, print_count: 0 });
        const user = await userRepo.create({ name: 'Test User' });
        const result = await resolvers.Mutation.printTask(
          null,
          { id: task.id.toString(), userId: user.id.toString() },
          context
        );
        if (!result) throw new Error('printTask mutation did not return a task');
        expect(result).not.toBeNull();
        expect(result && result.print_count).toBe(1);
        expect(result && result.last_printed_at).toBeDefined();
        expect(context.printer.printTask).toHaveBeenCalled();
      });

      it('should throw error for non-existent task', async () => {
        const user = await userRepo.create({ name: 'Test User' });
        await expect(
          resolvers.Mutation.printTask(null, { id: '999', userId: user.id.toString() }, context)
        ).rejects.toThrow('Task not found');
      });

      it('should throw error for non-existent user', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        const task = await taskRepo.create({ name: 'Test Task', group_id: group.id, print_count: 0 });
        await expect(
          resolvers.Mutation.printTask(null, { id: task.id.toString(), userId: '999' }, context)
        ).rejects.toThrow('User not found');
      });
    });

    describe('printStep', () => {
      it('should print a step', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        const task = await taskRepo.create({ name: 'Test Task', group_id: group.id, print_count: 0 });
        const step = await stepRepo.create({
          name: 'Test Step',
          instructions: 'Test Instructions',
          task_id: task.id,
          order: 1,
          print_count: 0
        });
        const user = await userRepo.create({ name: 'Test User' });
        const result = await resolvers.Mutation.printStep(
          null,
          { id: step.id.toString(), userId: user.id.toString() },
          context
        );
        if (!result) throw new Error('printStep mutation did not return a step');
        expect(result).not.toBeNull();
        expect(result && result.print_count).toBe(1);
        expect(result && result.last_printed_at).toBeDefined();
        expect(context.printer.printStep).toHaveBeenCalled();
      });

      it('should throw error for non-existent step', async () => {
        const user = await userRepo.create({ name: 'Test User' });
        await expect(
          resolvers.Mutation.printStep(null, { id: '999', userId: user.id.toString() }, context)
        ).rejects.toThrow('Step not found');
      });

      it('should throw error for non-existent user', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        const task = await taskRepo.create({ name: 'Test Task', group_id: group.id, print_count: 0 });
        const step = await stepRepo.create({
          name: 'Test Step',
          instructions: 'Test Instructions',
          task_id: task.id,
          order: 1,
          print_count: 0
        });
        await expect(
          resolvers.Mutation.printStep(null, { id: step.id.toString(), userId: '999' }, context)
        ).rejects.toThrow('User not found');
      });
    });
  });

  describe('Field Resolvers', () => {
    describe('Group', () => {
      it('should resolve tasks field', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        await taskRepo.create({ name: 'Task 1', group_id: group.id, print_count: 0 });
        await taskRepo.create({ name: 'Task 2', group_id: group.id, print_count: 0 });

        if (!group) throw new Error('Group not found');
        const result = await resolvers.Group.tasks(group, null, context);
        expect(result).toHaveLength(2);
        expect(result[0].name).toBe('Task 1');
        expect(result[1].name).toBe('Task 2');
      });
    });

    describe('Task', () => {
      it('should resolve group field', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        const task = await taskRepo.create({ name: 'Test Task', group_id: group.id, print_count: 0 });

        expect(task).not.toBeNull();
        const result = await resolvers.Task.group(task!, null, context);
        expect(result).not.toBeNull();
        expect(result!.name).toBe('Test Group');
      });

      it('should resolve steps field', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        const task = await taskRepo.create({ name: 'Test Task', group_id: group.id, print_count: 0 });
        await stepRepo.create({ name: 'Step 1', instructions: 'Instructions 1', task_id: task.id, order: 1, print_count: 0 });
        await stepRepo.create({ name: 'Step 2', instructions: 'Instructions 2', task_id: task.id, order: 2, print_count: 0 });

        expect(task).not.toBeNull();
        const result = await resolvers.Task.steps(task!, null, context);
        expect(result).toHaveLength(2);
        expect(result[0].name).toBe('Step 1');
        expect(result[1].name).toBe('Step 2');
      });

      it('should resolve notes field', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        const task = await taskRepo.create({ name: 'Test Task', group_id: group.id, print_count: 0 });
        const user = await userRepo.create({ name: 'Test User' });
        await noteRepo.create({ content: 'Note 1', task_id: task.id, created_by: user.id });
        await noteRepo.create({ content: 'Note 2', task_id: task.id, created_by: user.id });

        expect(task).not.toBeNull();
        const result = await resolvers.Task.notes(task!, null, context);
        expect(result).toHaveLength(2);
        expect(result[0].content).toBe('Note 1');
        expect(result[1].content).toBe('Note 2');
      });

      it('should resolve printHistory field', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        const task = await taskRepo.create({ name: 'Test Task', group_id: group.id, print_count: 0 });
        const user = await userRepo.create({ name: 'Test User' });
        await printHistoryRepo.create({
          user_id: user.id,
          task_id: task.id,
          printed_at: new Date(),
        });
        await printHistoryRepo.create({
          user_id: user.id,
          task_id: task.id,
          printed_at: new Date(),
        });

        expect(task).not.toBeNull();
        const result = await resolvers.Task.printHistory(task!, null, context);
        expect(result).toHaveLength(2);
      });
    });

    describe('Step', () => {
      it('should resolve task field', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        const task = await taskRepo.create({ name: 'Test Task', group_id: group.id, print_count: 0 });
        const step = await stepRepo.create({
          name: 'Test Step',
          instructions: 'Test Instructions',
          task_id: task.id,
          order: 1,
          print_count: 0
        });

        expect(step).not.toBeNull();
        const result = await resolvers.Step.task(step!, null, context);
        expect(result).not.toBeNull();
        expect(result!.name).toBe('Test Task');
      });

      it('should resolve images field', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        const task = await taskRepo.create({ name: 'Test Task', group_id: group.id, print_count: 0 });
        const step = await stepRepo.create({
          name: 'Test Step',
          instructions: 'Test Instructions',
          task_id: task.id,
          order: 1,
          print_count: 0
        });
        await imageRepo.create({
          step_id: step.id,
          original_path: '/path/to/original1.jpg',
          bw_path: '/path/to/bw1.jpg',
          order: 1,
        });
        await imageRepo.create({
          step_id: step.id,
          original_path: '/path/to/original2.jpg',
          bw_path: '/path/to/bw2.jpg',
          order: 2,
        });

        expect(step).not.toBeNull();
        const result = await resolvers.Step.images(step!, null, context);
        expect(result).toHaveLength(2);
        expect(result[0].original_path).toBe('/path/to/original1.jpg');
        expect(result[1].original_path).toBe('/path/to/original2.jpg');
      });

      it('should resolve notes field', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        const task = await taskRepo.create({ name: 'Test Task', group_id: group.id, print_count: 0 });
        const step = await stepRepo.create({
          name: 'Test Step',
          instructions: 'Test Instructions',
          task_id: task.id,
          order: 1,
          print_count: 0
        });
        const user = await userRepo.create({ name: 'Test User' });
        await noteRepo.create({ content: 'Note 1', step_id: step.id, created_by: user.id });
        await noteRepo.create({ content: 'Note 2', step_id: step.id, created_by: user.id });

        expect(step).not.toBeNull();
        const result = await resolvers.Step.notes(step!, null, context);
        expect(result).toHaveLength(2);
        expect(result[0].content).toBe('Note 1');
        expect(result[1].content).toBe('Note 2');
      });

      it('should resolve printHistory field', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        const task = await taskRepo.create({ name: 'Test Task', group_id: group.id, print_count: 0 });
        const step = await stepRepo.create({
          name: 'Test Step',
          instructions: 'Test Instructions',
          task_id: task.id,
          order: 1,
          print_count: 0
        });
        const user = await userRepo.create({ name: 'Test User' });
        await printHistoryRepo.create({
          user_id: user.id,
          step_id: step.id,
          printed_at: new Date(),
        });
        await printHistoryRepo.create({
          user_id: user.id,
          step_id: step.id,
          printed_at: new Date(),
        });

        expect(step).not.toBeNull();
        const result = await resolvers.Step.printHistory(step!, null, context);
        expect(result).toHaveLength(2);
      });
    });

    describe('Note', () => {
      it('should resolve createdBy field', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        const task = await taskRepo.create({ name: 'Test Task', group_id: group.id, print_count: 0 });
        const user = await userRepo.create({ name: 'Test User' });
        await noteRepo.create({
          content: 'Test Note',
          task_id: task.id,
          created_by: user.id,
        });

        const notes = await noteRepo.findByTaskId(task.id);
        expect(notes).toHaveLength(1);
        const result = await resolvers.Note.createdBy(notes[0], null, context);
        expect(result).not.toBeNull();
        expect(result!.name).toBe('Test User');
      });
    });

    describe('PrintHistory', () => {
      it('should resolve user field', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        const task = await taskRepo.create({ name: 'Test Task', group_id: group.id, print_count: 0 });
        const user = await userRepo.create({ name: 'Test User' });
        await printHistoryRepo.create({
          user_id: user.id,
          task_id: task.id,
          printed_at: new Date(),
        });

        const history = await printHistoryRepo.findByTaskId(task.id);
        expect(history).toHaveLength(1);
        const result = await resolvers.PrintHistory.user(history[0], null, context);
        expect(result).not.toBeNull();
        expect(result!.name).toBe('Test User');
      });

      it('should resolve task field', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        const task = await taskRepo.create({ name: 'Test Task', group_id: group.id, print_count: 0 });
        const user = await userRepo.create({ name: 'Test User' });
        await printHistoryRepo.create({
          user_id: user.id,
          task_id: task.id,
          printed_at: new Date(),
        });

        const history = await printHistoryRepo.findByTaskId(task.id);
        expect(history).toHaveLength(1);
        const result = await resolvers.PrintHistory.task(history[0], null, context);
        expect(result).not.toBeNull();
        expect(result!.name).toBe('Test Task');
      });

      it('should resolve step field', async () => {
        const group = await groupRepo.create({ name: 'Test Group' });
        const task = await taskRepo.create({ name: 'Test Task', group_id: group.id, print_count: 0 });
        const step = await stepRepo.create({
          name: 'Test Step',
          instructions: 'Test Instructions',
          task_id: task.id,
          order: 1,
          print_count: 0
        });
        const user = await userRepo.create({ name: 'Test User' });
        await printHistoryRepo.create({
          user_id: user.id,
          step_id: step.id,
          printed_at: new Date(),
        });

        const history = await printHistoryRepo.findByStepId(step.id);
        expect(history).toHaveLength(1);
        const result = await resolvers.PrintHistory.step(history[0], null, context);
        expect(result).not.toBeNull();
        expect(result!.name).toBe('Test Step');
      });
    });
  });
});