import { Knex } from 'knex';
import { GroupRepository, TaskRepository, StepRepository, UserRepository, NoteRepository, PrintHistoryRepository, ImageRepository } from '../db/repositories';
import type { Printer } from '@shared/index';

interface Context {
  db: Knex;
  printer: Printer;
}

export const resolvers = {
  Query: {
    groups: async (_: any, __: any, { db }: Context) => {
      const groupRepo = new GroupRepository(db);
      return await groupRepo.findAll();
    },
    group: async (_: any, { id }: { id: string }, { db }: Context) => {
      const groupRepo = new GroupRepository(db);
      return await groupRepo.findById(parseInt(id));
    },
    tasks: async (_: any, { groupId }: { groupId: string }, { db }: Context) => {
      const taskRepo = new TaskRepository(db);
      return await taskRepo.findByGroupId(parseInt(groupId));
    },
    task: async (_: any, { id }: { id: string }, { db }: Context) => {
      const taskRepo = new TaskRepository(db);
      return await taskRepo.findById(parseInt(id));
    },
    steps: async (_: any, { taskId }: { taskId: string }, { db }: Context) => {
      const stepRepo = new StepRepository(db);
      return await stepRepo.findByTaskId(parseInt(taskId));
    },
    step: async (_: any, { id }: { id: string }, { db }: Context) => {
      const stepRepo = new StepRepository(db);
      return await stepRepo.findById(parseInt(id));
    },
    recentTasks: async (_: any, __: any, { db }: Context) => {
      const taskRepo = new TaskRepository(db);
      return await taskRepo.findRecent();
    },
    frequentTasks: async (_: any, __: any, { db }: Context) => {
      const taskRepo = new TaskRepository(db);
      return await taskRepo.findFrequent();
    },
    users: async (_: any, __: any, { db }: Context) => {
      const userRepo = new UserRepository(db);
      return await userRepo.findAll();
    },
    user: async (_: any, { id }: { id: string }, { db }: Context) => {
      const userRepo = new UserRepository(db);
      return await userRepo.findById(parseInt(id));
    },
    notes: async (_: any, { stepId, taskId }: { stepId?: string; taskId?: string }, { db }: Context) => {
      const noteRepo = new NoteRepository(db);
      if (stepId) {
        return await noteRepo.findByStepId(parseInt(stepId));
      }
      if (taskId) {
        return await noteRepo.findByTaskId(parseInt(taskId));
      }
      return await noteRepo.findAll();
    },
    printHistory: async (_: any, { taskId, stepId }: { taskId?: string; stepId?: string }, { db }: Context) => {
      const printHistoryRepo = new PrintHistoryRepository(db);
      if (taskId) {
        return await printHistoryRepo.findByTaskId(parseInt(taskId));
      }
      if (stepId) {
        return await printHistoryRepo.findByStepId(parseInt(stepId));
      }
      return await printHistoryRepo.findAll();
    },
  },

  Mutation: {
    createGroup: async (_: any, { name, parentId }: { name: string; parentId?: string }, { db }: Context) => {
      const groupRepo = new GroupRepository(db);
      let parent_id: number | undefined = undefined;
      if (parentId) {
        const parent = await groupRepo.findById(parseInt(parentId));
        if (!parent) throw new Error('Parent group not found');
        parent_id = parseInt(parentId);
      }
      return await groupRepo.create({ name, parent_id });
    },
    createTask: async (_: any, { name, groupId }: { name: string; groupId: string }, { db }: Context) => {
      const groupRepo = new GroupRepository(db);
      const taskRepo = new TaskRepository(db);
      const group = await groupRepo.findById(parseInt(groupId));
      if (!group) throw new Error('Group not found');
      return await taskRepo.create({
        name,
        group_id: parseInt(groupId),
        print_count: 0,
      });
    },
    createStep: async (_: any, { name, instructions, taskId, order }: { name: string; instructions: string; taskId: string; order: number }, { db }: Context) => {
      const taskRepo = new TaskRepository(db);
      const stepRepo = new StepRepository(db);
      const task = await taskRepo.findById(parseInt(taskId));
      if (!task) throw new Error('Task not found');
      return await stepRepo.create({
        name,
        instructions,
        task_id: parseInt(taskId),
        order,
        print_count: 0,
      });
    },
    createImage: async (_: any, { stepId, originalPath, bwPath, order }: { stepId: string; originalPath: string; bwPath: string; order: number }, { db }: Context) => {
      const stepRepo = new StepRepository(db);
      const imageRepo = new ImageRepository(db);
      const step = await stepRepo.findById(parseInt(stepId));
      if (!step) throw new Error('Step not found');
      return await imageRepo.create({
        step_id: parseInt(stepId),
        original_path: originalPath,
        bw_path: bwPath,
        order,
      });
    },
    createUser: async (_: any, { name }: { name: string }, { db }: Context) => {
      const userRepo = new UserRepository(db);
      return await userRepo.create({ name });
    },
    createNote: async (
      _: any,
      { content, stepId, taskId, userId }: { content: string; stepId?: string; taskId?: string; userId: string },
      { db }: Context
    ) => {
      const userRepo = new UserRepository(db);
      const stepRepo = new StepRepository(db);
      const taskRepo = new TaskRepository(db);
      const noteRepo = new NoteRepository(db);
      const user = await userRepo.findById(parseInt(userId));
      if (!user) throw new Error('User not found');
      if (stepId) {
        const step = await stepRepo.findById(parseInt(stepId));
        if (!step) throw new Error('Step not found');
      } else if (taskId) {
        const task = await taskRepo.findById(parseInt(taskId));
        if (!task) throw new Error('Task not found');
      }
      return await noteRepo.create({
        content,
        step_id: stepId ? parseInt(stepId) : undefined,
        task_id: taskId ? parseInt(taskId) : undefined,
        created_by: parseInt(userId),
      });
    },
    printTask: async (_: any, { id, userId }: { id: string; userId: string }, { db, printer }: Context) => {
      const taskRepo = new TaskRepository(db);
      const userRepo = new UserRepository(db);
      const printHistoryRepo = new PrintHistoryRepository(db);
      const task = await taskRepo.findById(parseInt(id));
      if (!task) throw new Error('Task not found');
      const user = await userRepo.findById(parseInt(userId));
      if (!user) throw new Error('User not found');
      await printer.printTask(task, db);
      await printHistoryRepo.create({
        user_id: parseInt(userId),
        task_id: parseInt(id),
        printed_at: new Date(),
      });
      await taskRepo.update(parseInt(id), {
        print_count: (task.print_count || 0) + 1,
        last_printed_at: new Date(),
      });
      return await taskRepo.findById(parseInt(id));
    },
    printStep: async (_: any, { id, userId }: { id: string; userId: string }, { db, printer }: Context) => {
      const stepRepo = new StepRepository(db);
      const userRepo = new UserRepository(db);
      const printHistoryRepo = new PrintHistoryRepository(db);
      const step = await stepRepo.findById(parseInt(id));
      if (!step) throw new Error('Step not found');
      const user = await userRepo.findById(parseInt(userId));
      if (!user) throw new Error('User not found');
      await printer.printStep(step, db);
      await printHistoryRepo.create({
        user_id: parseInt(userId),
        step_id: parseInt(id),
        printed_at: new Date(),
      });
      await stepRepo.update(parseInt(id), {
        print_count: (step.print_count || 0) + 1,
        last_printed_at: new Date(),
      });
      return await stepRepo.findById(parseInt(id));
    },
  },

  Group: {
    tasks: async (group: { id: number }, _: any, { db }: Context) => {
      const taskRepo = new TaskRepository(db);
      return await taskRepo.findByGroupId(group.id);
    },
  },

  Task: {
    group: async (task: { group_id: number }, _: any, { db }: Context) => {
      const groupRepo = new GroupRepository(db);
      return await groupRepo.findById(task.group_id);
    },
    steps: async (task: { id: number }, _: any, { db }: Context) => {
      const stepRepo = new StepRepository(db);
      return await stepRepo.findByTaskId(task.id);
    },
    notes: async (task: { id: number }, _: any, { db }: Context) => {
      const noteRepo = new NoteRepository(db);
      return await noteRepo.findByTaskId(task.id);
    },
    printHistory: async (task: { id: number }, _: any, { db }: Context) => {
      const printHistoryRepo = new PrintHistoryRepository(db);
      return await printHistoryRepo.findByTaskId(task.id);
    },
  },

  Step: {
    task: async (step: { task_id: number }, _: any, { db }: Context) => {
      const taskRepo = new TaskRepository(db);
      return await taskRepo.findById(step.task_id);
    },
    images: async (step: { id: number }, _: any, { db }: Context) => {
      const imageRepo = new ImageRepository(db);
      return await imageRepo.findByStepId(step.id);
    },
    notes: async (step: { id: number }, _: any, { db }: Context) => {
      const noteRepo = new NoteRepository(db);
      return await noteRepo.findByStepId(step.id);
    },
    printHistory: async (step: { id: number }, _: any, { db }: Context) => {
      const printHistoryRepo = new PrintHistoryRepository(db);
      return await printHistoryRepo.findByStepId(step.id);
    },
  },

  Note: {
    createdBy: async (note: { created_by: number }, _: any, { db }: Context) => {
      const userRepo = new UserRepository(db);
      return await userRepo.findById(note.created_by);
    },
  },

  PrintHistory: {
    user: async (history: { user_id: number }, _: any, { db }: Context) => {
      const userRepo = new UserRepository(db);
      return await userRepo.findById(history.user_id);
    },
    task: async (history: { task_id?: number }, _: any, { db }: Context) => {
      if (!history.task_id) return null;
      const taskRepo = new TaskRepository(db);
      return await taskRepo.findById(history.task_id);
    },
    step: async (history: { step_id?: number }, _: any, { db }: Context) => {
      if (!history.step_id) return null;
      const stepRepo = new StepRepository(db);
      return await stepRepo.findById(history.step_id);
    },
  },
};